@@ -217,9 +217,12 @@ class IndexConstraints(ReportNodeFunc):
             yield shape_edit
             return
 
-        # each usage should have a single entry
-        self.render.set_inputs(obs_args, shape_edit.index_ranks,
-                shape_edit.layout)
+        dims_inputs = { 
+                **obs_args, 
+                base.INDEX_RANKS: shape_edit.index_ranks,
+                base.LAYOUT: shape_edit.layout 
+                }
+        self.render.set_inputs(dims_inputs)
 
         input_dims = shape_edit.get_input_dims(use_scalars=True)
         comp_dims = self.render.get_dims(input_dims)
@@ -234,9 +237,8 @@ class IndexConstraints(ReportNodeFunc):
             if not all(idx in index_dims for idx in pred.indices):
                 continue
             pred_input_dims = [ index_dims[idx] for idx in pred.indices ]
-            # resolve argnames
-            argvals = tuple(obs_args[v] for v in pred.arg_names)
-            if not pred(*pred_input_dims, *argvals):
+
+            if not pred(*pred_input_dims):
                 # collect the predecessor formulas
                 comp_nodes = self.op._comp_dims_nodes()
                 comp_names = [ n.sub_name for n in comp_nodes ]
