@@ -453,31 +453,38 @@ class OpSchema(object):
                 nodes.append(node)
         return nodes
 
+    def _dims_arg_nodes(self):
+        """
+        Return a map of arg_name => node, which includes all generative nodes
+        used as inputs to any computed dims.  Although INDEX_RANKS is one of
+        the DimsInput nodes, this is not included since it is never an input
+        for a CompDims node
+        """
+        gen_nodes = {}
+        for node in self.dims_graph.values():
+            func = node.func
+            if isinstance(func, ge.DimsInput) and func.gen_node is not None:
+                gen_node = func.gen_node
+                gen_nodes[gen_node.sub_name] = gen_node
+        return gen_nodes
+
     def comp_dims_report(self, use_full_names=False):
         """
         Show a listing of all computed index dimensions 
         """
-        comp_nodes = self._comp_dims_nodes() 
-        if len(comp_nodes) == 0:
-            return None
-
+        gen_nodes_map = self._dims_arg_nodes()
+        gen_nodes = gen_nodes_map.values()
         render = base.RenderCompDims(self)
-        inp_nodes = self._dims_input_nodes()
-        inp_nodes = [n for n in inp_nodes if n.sub_name != base.INDEX_RANKS]
-        inp_names = [n.sub_name for n in inp_nodes]
-        self.dims_graph_input[base.INDEX_RANKS] = {}
-
-        # The generative nodes outside the comp_dims graph
-        gen_nodes = [n.func.gen_node for n in inp_nodes if n.func.gen_node is
-                not None]
 
         comp_formulas = {}
-        vals_list = fgraph.all_values(*gen_nodes)
-        for vals in vals_list:
-            for name, val in zip(inp_names, vals):
-                if isinstance(val, OpArg):
-                    val = val.value()
-                self.dims_graph_input[name] = val
+        gen = fgraph.gen_graph_map(gen_nodes, gen_nodes, full_name=False)
+        dims_inputs_list = list(gen)
+        if len(dims_inputs_list) == 0:
+            dims_inputs_list = [{}]
+
+        for dims_inputs in dims_inputs_list:
+            render.set_inputs(dims_inputs)
+            render.set_inputs({ base.INDEX_RANKS: {} })
             if use_full_names:
                 eq_map = render.get_snake()
             else:
@@ -487,6 +494,9 @@ class OpSchema(object):
                 eqn = f'{lhs} = {rhs}'
                 cset.add(eqn)
 
+        if len(comp_formulas) == 0:
+            return None
+
         formulas = []
         for idx, cset in comp_formulas.items():
             formulas.extend(cset)
@@ -494,6 +504,23 @@ class OpSchema(object):
         report = '\n'.join(formulas)
         return report
 
+    def index_preds_report(self):
+        """
+        Show a listing of index predicates
+        """
+        if len(self.index_preds) == 0:
+            return None
+
+        snakes = []
+        olcs = []
+        for pred in self.index_preds:
+            snake = pred.get_formula(self, snake_case=True)
+            olc = pred.get_formula(self, snake_case=False)
+            snakes.append(snake)
+            olcs.append(olc)
+        report = '\n'.join(snakes) + '\n\n' + '\n'.join(olcs)
+        return report
+
     def dtype_rules_report(self):
         """
         Return a report of rules defining the set of all allowed tensor dtype
@@ -579,6 +606,7 @@ class OpSchema(object):
         index_ranks = self.index_ranks_report()
         comp_dims_snake = self.comp_dims_report(True)
         comp_dims_olc = self.comp_dims_report(False)
+        index_preds = self.index_preds_report()
         dtype_rules = self.dtype_rules_report()
         combo_rules = self.excluded_dtypes_report()
 
@@ -594,6 +622,11 @@ class OpSchema(object):
         else:
             finals.append(f'Computed dimensions\n\n{comp_dims_snake}\n\n{comp_dims_olc}')
 
+        if index_preds is None:
+            finals.append(f'Index predicates\n\nNone')
+        else:
+            finals.append(f'Index predicates\n\n{index_preds}')
+
         finals.append(f'DType Rules\n\n{dtype_rules}')
 
         if combo_rules is not None:
@@ -751,12 +784,14 @@ class OpSchema(object):
         for ind in self.index.values():
             if ind.dims_node_cls is None:
                 raise SchemaError(
+                    f'Schema {self.op_path} contains error:\n'
                     f'Index {ind.idx} has not been registered with either '
                     f'gen_dims_* or comp_dims_* API calls.  All indexes '
                     f'must be registered with one of these.')
             if ind.is_computed():
                 if ind.has_insig:
                     raise SchemaError(
+                        f'Schema {self.op_path} contains error:\n'
                         f'Index {ind.idx} is a computed index (registered '
                         f'with comp_dims* API call) but also appears in one '
                         f'or more input signatures.  Only non-computed indexes '
@@ -764,6 +799,7 @@ class OpSchema(object):
             else:
                 if not ind.has_insig:
                     raise SchemaError(
+                        f'Schema {self.op_path} contains error:\n'
                         f'Index {ind.idx} is a non-computed index '
                         f'(registered with gen_dims* API call) but does not '
                         f'appear in any input signature.  All non-computed '
@@ -792,7 +828,7 @@ class OpSchema(object):
         for op_args in fgraph.gen_graph_values(live, out, self):
             yield op_args[0] # extract tuple element
 
-    def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota):
+    def validate(self, out_dir, test_ids, skip_ids, dtype_err_quota, rand_seed):
         if not os.path.exists(out_dir):
             raise RuntimeError(
                 f'{type(self).__qualname__}: Could not open output path '
@@ -805,9 +841,7 @@ class OpSchema(object):
         cats = [ 'TP', 'TN', 'FP', 'FN' ]
         stats = { k: 0 for k in cats }
 
-        op_args_gen = self.generate_args()
-        # op_args_gen = list(self.generate_args())
-        # print(f'Generated {len(op_args_list)} test cases')
+        op_args_gen = self.generate_args(rand_seed)
 
         for test_id, op_args in enumerate(op_args_gen, 1):
             if skip_ids is not None and test_id in skip_ids:
@@ -1047,7 +1081,7 @@ class OpSchema(object):
                 f'the dims_graph with one of the gen_dims_* or comp_dims '
                 f'API functions')
 
-    def gen_dims_func(self, out_sig, func, in_sig, max_prod, do_scalar, *pars):
+    def gen_dims_func(self, out_sig, func, in_sig, max_prod, do_scalar, *arg_names):
         """
         Calls func(gen_rng, *in_dims, *pars), which yields one or more
         results.  Each result is either a tuple (lo, hi), if out_sig is a
@@ -1065,19 +1099,39 @@ class OpSchema(object):
         try:
             self._check_out_sig(out_sig)
         except SchemaError as ex:
-            raise SchemaError(f'gen_dims_func got error: {ex}')
+            raise SchemaError(f'Error constructing gen_dims for {out_sig}:\n{ex}')
 
+        mut_gnode = self._gen_node(ge.ArgMutations)
         for idx in out_sig:
             ind = self.index[idx]
             ind.dims_node_cls = ge.GenDims
 
+        arg_parents = []
+        for arg_name in arg_names:
+            dnode = self._maybe_add_arg_parent(mut_gnode, arg_name)
+            arg_parents.append(dnode)
+
         pri_idx = self._get_rank_idx(in_sig, out_sig)
-        gdims = ge.GenDims(self, out_sig, in_sig, func, pri_idx, do_scalar,
-                max_prod, pars)
+        gdims = ge.GenDims(self, out_sig, in_sig, func, pri_idx, do_scalar, max_prod,
+            *arg_names)
         G.set_registry(self.dims_graph)
         parents = self._get_dims_nodes(in_sig)
         ranks_dnode = self._dims_node(ge.DimsInput, base.INDEX_RANKS)
-        G.add_node_sn(gdims, ranks_dnode, *parents)
+        G.add_node_sn(gdims, ranks_dnode, *parents, *arg_parents)
+
+    def gen_dims_calc(self, out_sig, func, in_sig, *arg_names):
+        """
+        Calls func(*in_dims, *arg_vals) which yields one or more results.
+        Each result is either a tuple (lo, hi), if out_sig is a single index,
+        or a tuple of tuples, one for each index in out_sig.
+
+        `arg_names` must be names of arguments passed to the op
+        arg_vals are the runtime resolved values of the argument names
+        """
+        def gen(rng, *args):
+            yield from func(*args)
+
+        return self.gen_dims_func(out_sig, gen, in_sig, int(1e10), False, *arg_names)
 
     def gen_dims(self, out_idx, max_prod):
         """
@@ -1128,34 +1182,47 @@ class OpSchema(object):
         """
         return self._comp_dims(out_idx, func, tfunc, in_sig, True, *arg_names)
 
+    def _maybe_add_arg_parent(self, node, arg_name):
+        if arg_name == base.LAYOUT:
+            arg_gnode = self._gen_node(ge.Layout, base.LAYOUT) 
+            node.maybe_append_parent_sn(arg_gnode)
+
+        elif arg_name in self.arg_order:
+            arg_gnode = self.arg_gen_nodes[arg_name]
+            node.maybe_append_parent_sn(arg_gnode)
+
+        else:
+            raise SchemaError(
+                f'name \'{arg_name}\' in arg_names must be either a '
+                f'framework op argument, or the constant '
+                f'\'{base.LAYOUT}\'.\n'
+                f'Input arguments are: ' + ', '.join(self.arg_order))
+
+        dnode = self._dims_node(ge.DimsInput, arg_name)
+        if dnode is None:
+            dobj = ge.DimsInput(self, arg_name)
+            dnode = G.add_node(dobj)
+        return dnode
+
     def _comp_dims(self, out_idx, func, tfunc, in_sig, cwise, *arg_names):
+        """
+        Instantiate `out_idx` as a computed index.  The dimensions are computed
+        as func(*in_dims, *arg_vals).  If `cwise`, dimensions are computed
+        component-wise: func is called once per component.
+
+        arg_vals are the run-time argument values resolved from `arg_names`
+
+        tfunc is the corresponding template function, which produces
+        human-readable formulae of the computation.
+        """
         self._check_out_sig(out_idx)
         mut_gnode = self._gen_node(ge.ArgMutations)
 
         G.set_registry(self.dims_graph)
 
         arg_parents = []
-
         for arg_name in arg_names:
-            if arg_name == base.LAYOUT:
-                arg_gnode = self._gen_node(ge.Layout, base.LAYOUT) 
-                mut_gnode.maybe_append_parent_sn(arg_gnode)
-
-            elif arg_name in self.arg_order:
-                arg_gnode = self.arg_gen_nodes[arg_name]
-                mut_gnode.maybe_append_parent_sn(arg_gnode)
-
-            else:
-                raise SchemaError(
-                    f'name \'{arg_name}\' in arg_names must be either a '
-                    f'framework op argument, or the constant '
-                    f'\'{base.LAYOUT}\'.\n'
-                    f'Input arguments are: ' + ', '.join(self.arg_order))
-
-            dnode = self._dims_node(ge.DimsInput, arg_name)
-            if dnode is None:
-                dobj = ge.DimsInput(self, arg_name)
-                dnode = G.add_node(dobj)
+            dnode = self._maybe_add_arg_parent(mut_gnode, arg_name)
             arg_parents.append(dnode)
 
         ind = self.index[out_idx]
@@ -1640,7 +1707,7 @@ class OpSchema(object):
             inode = self.inf_graph[pri_idx]
             inode.func.add_shapes_constraint(cons)
 
-    def dims_pred(self, pred_name, pfunc, pfunc_t, indices, *arg_names):
+    def dims_pred(self, pred_name, pfunc, pfunc_t, indices):
         """
         Registers {pfunc} with the schema to be used as an additional
         predicate for {indexes} dimensions.
@@ -1652,21 +1719,24 @@ class OpSchema(object):
         {pred_name} is a name given to this custom predicate.  It may be used
         in error messages.
 
-        Called as pfunc(*index_shapes, *argvals), where index_shapes
-        are the resolved shapes of `indices` and argvals are the resolved
-        values of `arg_names`.
+        Called as pfunc(*index_shapes), where index_shapes
+        are the resolved shapes of `indices`.
+
+        Note: dims predicate functions can only operate on index dimensions.
+        To model a predicate that depends on op parameters as well, first
+        create an intermediate computed index with add_index and comp_dims API
+        calls, which accept non-index parameters.  Then, use that intermediate
+        index in a predicate.
         """
-        pred = base.IndexPredicate(pred_name, False, pfunc, pfunc_t, indices,
-                *arg_names)
+        pred = base.IndexPredicate(pred_name, False, pfunc, pfunc_t, indices)
         self.index_preds.append(pred)
 
-    def dims_pred_cw(self, pred_name, pfunc, pfunc_t, indices, *arg_names):
+    def dims_pred_cw(self, pred_name, pfunc, pfunc_t, indices):
         """
         Like dims_pred, but pfunc is called 'component-wise'.  That is, it is
         called once for each set of broadcasted index shapes.
         """
-        pred = base.IndexPredicate(pred_name, True, pfunc, pfunc_t, indices,
-                *arg_names)
+        pred = base.IndexPredicate(pred_name, True, pfunc, pfunc_t, indices)
         self.index_preds.append(pred)
 
     def dims_pred_rng(self, idx, lo, hi):
